<?php
namespace lib\db;
use core;

include("./base.php");

use lib\db\Base;

class Assembly extends Base
{
	public function __construct() 
    {
		$types = 
		[
			"assembly",
			"part",
			"fastener",
			"service"
		];
		parent::__construct("assembly",$types);
	}

	function prepareData($input)
    {
        if(!$input) return $input;
        
        $data = core\getVal($input,"data");

        $cols =  core\getVal($input,"cols", ["id","quote_id","name"]);

		$rowData = array_filter($data,	function ($key) use ($cols) {return in_array($key, $cols);},	ARRAY_FILTER_USE_KEY);
		foreach($cols as $key)
		{
			unset($data[$key]);
		}
		$rowData["data"] = json_encode($data);
        return $rowData;
    }

	function  build()
	{
		$fields=[];
		array_push($fields,"name VARCHAR(256) NOT NULL");
		array_push($fields,"quote_id INT(64) UNSIGNED NOT NULL");
		array_push($fields,"data JSON");
		$res = $this->sql-> buildTable($this->table,$fields);
        $res = json_encode($res);
    }

	function get_linked($input = [])
	{
		$qty = core\getVal($input,"unit",1);

		$linked = $this->link->getLinked1($input);
		

		$data = core\getVal($linked,"data",[]);
		foreach($data as &$sub)
		{
			$s_qty = core\getVal($sub,"unit",0);
			$_qty =  $s_qty*$qty;
			$sub["unit"] = $_qty;
			$sub["desc"] = "Qty/Assy = $s_qty ; Total Qty = $_qty";
		}
		$linked["data"]= $data; 
		return $linked;
	}

	function update_qty($input=[])
	{
		$data = core\getVal($input,"data",0);

		$qty = core\getVal($data,"unit",0);

		$req_view = [];
		$req_view["id"] =  core\getVal($data,"link_id",0);
		$res_view = $this->link->view($req_view);

		$sub =  core\getVal($res_view,array_key_first($res_view),[]);
		
		if($sub) $sub["unit"] = $qty;
		
		$req_link = [];
		$req_link["data"] = $sub;
		
		return $this->link->save($req_link);
	}
}

$e = new Assembly();