<?php
namespace lib\db;

include_once("../lib/f_proxy.php");
p_include("core/php/mysql2.php","");
p_include("core/php/util.php","");

use core;
use core\MYSQL;

class Link
{
    public $table;
    public $parent_type;
    public $child_types;
	public function __construct($parent_type,$child_types=[]) 
    {
        $this->table = "link";
        $this->parent_type = $parent_type;
        $this->child_types = $child_types;
        $settings = core\getSettings("../lib"); 
        $db=core\getVal($settings,"db");
        $this->sql = new MYSQL($db);

        $this->build();
    }

	function  build()
	{
		$fields=[];
		array_push($fields,"parent_id INT(64) NOT NULL");
		array_push($fields,"parent_type VARCHAR(256) NOT NULL");
		array_push($fields,"child_id INT(64) NOT NULL");
		array_push($fields,"child_type VARCHAR(256) NOT NULL");
		array_push($fields,"unit INT(64)");
		array_push($fields,"data JSON");
		$res = $this->sql-> buildTable($this->table,$fields);
        $res = json_encode($res);
    }

	function prepareData($input=[])
    {
        if(!$input) return $input;
        
        $data = core\getVal($input,"data");
        $data["parent_type"] = $this->parent_type;
        $cols =  core\getVal($input,"cols", ["id","parent_type","parent_id","child_type","child_id","unit"]);

		$rowData = array_filter($data,	function ($key) use ($cols) {return in_array($key, $cols);},	ARRAY_FILTER_USE_KEY);
		foreach($cols as $key)
		{
			unset($data[$key]);
		}
        $rowData["data"] = json_encode($data);
        return $rowData;
    }

	function save($input=[])
    {
		$input = $this->prepareData($input);
		
        $req = 
        [
			"table" => $this->table,
            "data" => $input,
        ];
		
        $res = $this->sql->save($req);

        $data = core\getVal($res,'data',[]);
        $json = core\getVal($data,'data',"");
        $res["data"]["data"] = json_decode($json);
        return core\getVal($res,'data',[]);
    }

    function view($input=[])
    {
        $id = core\getVal($input,"id");
        $where = core\getVal($input,"where");
        if(!$where && $id)
        {
            $req["where"] = "WHERE id = $id";
        } 
        else $req["where"] = $where;


        $req["table"] =$this->table;
        $req["limit"] = 1;
        $res = $this->sql->getRows($req);

        return core\getVal($res,'data',[]);

    }

	function remove($input=[])
    {
        $input["table"] = $this->table;
        $res = $this->sql->delete($input);
        return core\getVal($res,'data',[]);

    }

    function getLinked1($input = [])
    {
        $rows = [];

        $req = [];
        $p_type = core\getVal($input,"parent_type",$this->parent_type); ;
       
        $p_id = core\getVal($input,"parent_id",0);
        $c_type = core\getVal($input,"child_type",false);
        
        $req["table"]  = $this->table;
        $c_types = core\getVal($input,"child_types",$this->child_types);
        if($c_type)
        {
            // $c_types=[];
            $c_types[]=$c_type;
        }
       
        if($c_types)
        {
            foreach($c_types as $type)
            {
                $fields = [];
                $fields[] = "link.data as link";
                $fields[] = "link.id as link_id";
                $fields[] = "link.unit as unit";
                $fields[] = "$type.id as id ";
                $fields[] = "$type.name as name ";
                $fields[] = "'$type' as type" ;
                $fields[] = "$type.data as data ";
                $join = "JOIN $type on $type.id = link.child_id ";
                $req["join"] = $join;
                $req["fields"] = $fields ;
                $req["where"] = "WHERE parent_type = '$p_type' AND parent_id = $p_id AND child_type = '$type'" ;
               
                $req["parent_type"] = $this->parent_type;
                $res_ = $this->sql->getRows($req);
              
                if($res_!=null) $rows = array_merge_recursive($rows,$res_);

            }
        }
        return $rows;
    }

	function getParents($input = [])
	{
        $result = [];

        $c_id = core\getVal($input,"child_id",0);
        $c_type = core\getVal($input,"child_type",0);

        $conn_request = $this->sql->getConnection();
        $conn = $conn_request["conn"];
        if(isset($conn_request['error'])) $result['error'] = $conn_request['error']; 
        else
        {
            $sql = 
            "
                SELECT
                    parent_id as id,
                    parent_type as type
                FROM
                    link
                WHERE child_id = $c_id AND child_type = '$c_type' AND parent_type = '$c_type'
            ";


            $res = $conn->query($sql);

            $row_count = $res? $res->num_rows:0;

            $result["found"] = [];

            if ($row_count > 0) 
            {
                $res_data = mysqli_fetch_all ($res, MYSQLI_ASSOC);

                if($res_data)
                {
                    foreach ($res_data as $value) 
                    {
                        $result["found"][]= $value;
                        $test["child_type"] = core\getVal($value,"type",0);
                        $test["child_id"] = core\getVal($value,"id",0);
                        $rec_res = $this->getParents($test);
                        $found = core\getVal($rec_res,"found",0);
                        $result["found"] = array_merge($result["found"],$found);
                    }
                }
            }

            $conn->close();

            return $result;
        }
	}

}