<?php
namespace lib\db;

include("./auth.php");
include("./base.php");

use lib\db\Base;
use core;

class User extends Base
{
    public function __construct() 
    {
		parent::__construct("user");
	}

    function  build()
	{
		$fields=[];
		array_push($fields,"email VARCHAR(256) NOT NULL");
		array_push($fields,"pass VARCHAR(256) NOT NULL");
		array_push($fields,"auth_level INT(64) UNSIGNED NOT NULL DEFAULT 0");
		array_push($fields,"data JSON");
		$res = $this->sql-> buildTable($this->table,$fields);
        $res = json_encode($res);
    }

    function prepareData($input=[])
    {
        if(!$input) return $input;
        
        $data = core\getVal($input,"data");

        $cols =  core\getVal($input,"cols", ["id","email","pass","auth_level"]);

		$rowData = array_filter($data,	function ($key) use ($cols) {return in_array($key, $cols);},	ARRAY_FILTER_USE_KEY);
		foreach($cols as $key)
		{
			unset($data[$key]);
		}
        $rowData["data"] = json_encode($data);
        return $rowData;
    }

    function save($input = null)
    {
        $data = core\getVal($input,"data");
        $pass = core\getVal($data,"pass");

        if($pass)
        {
            $pass = urlencode($pass);
            $input["data"]["pass"] = password_hash($pass,PASSWORD_DEFAULT);
        }

        $res = parent::save($input);
        $id = core\getVal($res,"id",false);
        $row["status"]=$id!=false;
        return $row;
    }

    function edit($input = null)
    {
        return parent::save($input);
    }

    function userLogin($input=[])
    {
        $data = core\getVal($input,"data");
        $pass = core\getVal($data,"pass");
        $email = core\getVal($data,"email");

        $req["where"] = "WHERE email = '$email'";
        $res = parent::list($req);

        $rows = core\getVal($res,"data",[]);;

        $pass = urlencode($pass);
        $res = [];
       
        foreach($rows as $row)
        {
            $pass_ = core\getVal($row,"pass");
           
            if (password_verify($pass, $pass_)) 
            {
                $req = 
                [
                    "user_id"=> core\getVal($row,"id")
                ];

                $res = (new Auth())->get($req);

                // print_r($res);
                if($res)
                {
                    $res = core\getVal($res,'data',[]);
                }
                
                break;
            } 
        }
        return $res;
    }

    function userLogout($input=[])
    {
        $req["hash"]= core\getVal($input,"hash");

        $res = (new Auth())->endAuth($req);

        return core\getVal($res,'data',[]);
    }

    function getUser($input=[])
    {
        $auth = core\getVal($input,"auth");

        if(!$auth) return null;
        $req["fields"] = 
        [
            "user.email",
            "auth.hash as hash",
			"user.data as data"
        ];
        $req["join"] = "JOIN auth on user_id = user.id";
        $req["where"] = "WHERE auth.hash = '$auth' AND auth.end is null";
        $req["table"] =$this->table;
        $req["limit"] = 1;
        $res = $this->sql->getRows($req);

        $rows = core\getVal($res,'data',[]);
        if($rows) 
        {
            $row = core\getVal($rows,array_key_first($rows));

            return $row;
            
        }

        return null;
       
    }

    function getLevel($input=[])
    {
        $hash = core\getVal($input,"hash");
        $req["where"] = "WHERE auth.hash = '$hash' AND auth.end is null" ;
        $req["fields"] = ["user.auth_level"];
        $req["join"] = "JOIN auth on user_id = user.id";
        $req["table"] =$this->table;
        $req["limit"] = 1;
        $res = $this->sql->getRows($req);

        $data = core\getVal($res,'data',[]);

        return core\getVal($data,array_key_first($data));

    }

    function checkEmail($input=[])
    {
        $email = core\getVal($input,"email");
        $req["where"] = "WHERE email = '$email'";
        $req["fields"] = ["email"];
        $req["table"] =$this->table;
        $req["limit"] = 1;
        $res = $this->sql->getRows($req);

        $data = core\getVal($res,'data',[]);

        return core\getVal($data,array_key_first($data));

    }
    
    function list($input=[])
    {
        $input["search_fields"] = ["email","name"];
        // $input["fields"] = ["email","auth_level","id", "JSON_UNQUOTE(data->'$.image') as image","data->>'$.name' as name"];
        $res = parent::list($input);
        $rows = core\getVal($res,'data',[]);
        if($rows) 
        {
            $cols =  core\getVal($input,"cols", ["pass"]);

            foreach($rows as &$row)
            {

                foreach($cols as $key)
                {
                    unset($row[$key]);
                }
            }
           
        }

        // $res['data'] = $rows;
        return $rows;
    }
}

$e = new User();